@@ -27,9 +27,11 @@ from code import Code
 _ZERO = "\x00\x00\x00\x00"
 _ONE = "\x01\x00\x00\x00"
 
+
 class Collection(object):
     """A Mongo collection.
     """
+
     def __init__(self, database, name, options=None):
         """Get / create a Mongo collection.
 
@@ -85,7 +87,8 @@ class Collection(object):
         :Parameters:
           - `name`: the name of the collection to get
         """
-        return Collection(self.__database, u"%s.%s" % (self.__collection_name, name))
+        return Collection(self.__database, u"%s.%s" % (self.__collection_name,
+                                                       name))
 
     def __getitem__(self, name):
         return self.__getattr__(name)
@@ -145,10 +148,12 @@ class Collection(object):
             result = self.insert(to_save, manipulate, safe)
             return result.get("_id", None)
         else:
-            self.update({"_id": to_save["_id"]}, to_save, True, manipulate, safe)
+            self.update({"_id": to_save["_id"]}, to_save, True,
+                        manipulate, safe)
             return to_save.get("_id", None)
 
-    def insert(self, doc_or_docs, manipulate=True, safe=False):
+    def insert(self, doc_or_docs,
+               manipulate=True, safe=False, check_keys=True):
         """Insert a document(s) into this collection.
 
         If manipulate is set the document(s) are manipulated using any
@@ -162,6 +167,8 @@ class Collection(object):
           - `doc_or_docs`: a SON object or list of SON objects to be inserted
           - `manipulate` (optional): monipulate the objects before inserting?
           - `safe` (optional): check that the insert succeeded?
+          - `check_keys` (optional): check if keys start with '$' or
+            contain '.', raising `pymongo.errors.InvalidName` in either case
         """
         docs = doc_or_docs
         if isinstance(docs, types.DictType):
@@ -173,7 +180,7 @@ class Collection(object):
         if manipulate:
             docs = [self.__database._fix_incoming(doc, self) for doc in docs]
 
-        data = [bson.BSON.from_dict(doc) for doc in docs]
+        data = [bson.BSON.from_dict(doc, check_keys) for doc in docs]
         self._send_message(2002, "".join(data))
 
         if safe:
@@ -183,7 +190,8 @@ class Collection(object):
 
         return len(docs) == 1 and docs[0] or docs
 
-    def update(self, spec, document, upsert=False, manipulate=False, safe=False):
+    def update(self, spec, document,
+               upsert=False, manipulate=False, safe=False):
         """Update an object(s) in this collection.
 
         Raises TypeError if either spec or document isn't an instance of
@@ -193,8 +201,8 @@ class Collection(object):
         database.
 
         :Parameters:
-          - `spec`: a SON object specifying elements which must be present for a
-            document to be updated
+          - `spec`: a SON object specifying elements which must be present for
+            a document to be updated
           - `document`: a SON object specifying the fields to be changed in the
             selected document(s), or (in the case of an upsert) the document to
             be inserted.
@@ -231,15 +239,16 @@ class Collection(object):
 
         :Parameters:
           - `spec_or_object_id` (optional): a SON object specifying elements
-            which must be present for a document to be removed OR an instance of
-            ObjectId to be used as the value for an _id element
+            which must be present for a document to be removed OR an instance
+            of ObjectId to be used as the value for an _id element
         """
         spec = spec_or_object_id
         if isinstance(spec, ObjectId):
             spec = SON({"_id": spec})
 
         if not isinstance(spec, types.DictType):
-            raise TypeError("spec must be an instance of dict, not %s" % type(spec))
+            raise TypeError("spec must be an instance of dict, not %s" %
+                            type(spec))
 
         self._send_message(2006, _ZERO + bson.BSON.from_dict(spec))
 
@@ -265,6 +274,33 @@ class Collection(object):
             return result
         return None
 
+    def _fields_list_to_dict(self, fields):
+        """Takes a list of field names and returns a matching dictionary.
+
+        ["a", "b"] becomes {"a": 1, "b": 1}
+
+        and
+
+        ["a.b.c", "d", "a.c"] becomes {"a": {"b": {"c": 1}, "c": 1}, "d": 1}
+        """
+        as_dict = {}
+        for field in fields:
+            if not isinstance(field, types.StringTypes):
+                raise TypeError("fields must be a list of key names as "
+                                "(string, unicode)")
+            keys = field.split(".")
+
+            base = as_dict
+            while len(keys):
+                key = keys.pop(0)
+                if key not in base:
+                    if len(keys):
+                        base[key] = {}
+                    else:
+                        base[key] = 1
+                base = base[key]
+        return as_dict
+
     def find(self, spec=None, fields=None, skip=0, limit=0, _sock=None):
         """Query the database.
 
@@ -286,7 +322,7 @@ class Collection(object):
           - `spec` (optional): a SON object specifying elements which must be
             present for a document to be included in the result set
           - `fields` (optional): a list of field names that should be returned
-            in the result set
+            in the result set ("_id" will always be included)
           - `skip` (optional): the number of documents to omit (from the start
             of the result set) when returning the results
           - `limit` (optional): the maximum number of results to return in the
@@ -303,15 +339,10 @@ class Collection(object):
         if not isinstance(limit, types.IntType):
             raise TypeError("limit must be an instance of int")
 
-        return_fields = None
         if fields is not None:
-            return_fields = {}
-            for field in fields:
-                if not isinstance(field, types.StringTypes):
-                    raise TypeError("fields must be a list of key names as (string, unicode)")
-                return_fields[field] = 1
+            fields = self._fields_list_to_dict(fields)
 
-        return Cursor(self, spec, return_fields, skip, limit, _sock=_sock)
+        return Cursor(self, spec, fields, skip, limit, _sock=_sock)
 
     def count(self):
         return self.find().count()
@@ -324,10 +355,10 @@ class Collection(object):
     def create_index(self, key_or_list, direction=None, unique=False, ttl=300):
         """Creates an index on this collection.
 
-        Takes either a single key and a direction, or a list of (key, direction)
-        pairs. The key(s) must be an instance of (str, unicode), and the
-        direction(s) must be one of (`pymongo.ASCENDING`, `pymongo.DESCENDING`).
-        Returns the name of the created index.
+        Takes either a single key and a direction, or a list of (key,
+        direction) pairs. The key(s) must be an instance of (str, unicode),
+        and the direction(s) must be one of (`pymongo.ASCENDING`,
+        `pymongo.DESCENDING`). Returns the name of the created index.
 
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction) pairs
@@ -351,15 +382,17 @@ class Collection(object):
                                                   self.name(),
                                                   name, ttl)
 
-        self.database().system.indexes.save(to_save, False)
+        self.database().system.indexes.insert(to_save, manipulate=False,
+                                              check_keys=False)
         return to_save["name"]
 
     def ensure_index(self, key_or_list, direction=None, unique=False, ttl=300):
         """Ensures that an index exists on this collection.
 
-        Takes either a single key and a direction, or a list of (key, direction)
-        pairs. The key(s) must be an instance of (str, unicode), and the
-        direction(s) must be one of (`pymongo.ASCENDING`, `pymongo.DESCENDING`).
+        Takes either a single key and a direction, or a list of (key,
+        direction) pairs. The key(s) must be an instance of (str, unicode),
+        and the direction(s) must be one of (`pymongo.ASCENDING`,
+        `pymongo.DESCENDING`).
 
         Unlike `create_index`, which attempts to create an index
         unconditionally, `ensure_index` takes advantage of some caching within
@@ -400,7 +433,8 @@ class Collection(object):
         Can be used on non-existant collections or collections with no indexes.
         Raises OperationFailure on an error.
         """
-        self.database().connection()._purge_index(self.database().name(), self.name())
+        self.database().connection()._purge_index(self.database().name(),
+                                                  self.name())
         self.drop_index(u"*")
 
     def drop_index(self, index_or_name):
@@ -422,7 +456,8 @@ class Collection(object):
         if not isinstance(name, types.StringTypes):
             raise TypeError("index_or_name must be an index name or list")
 
-        self.database().connection()._purge_index(self.database().name(), self.name(), name)
+        self.database().connection()._purge_index(self.database().name(),
+                                                  self.name(), name)
         self.__database._command(SON([("deleteIndexes",
                                        self.__collection_name),
                                       ("index", name)]),
@@ -494,10 +529,11 @@ class Collection(object):
     }
     return {"result": map.values()};
 }""" % reduce
-        return self.__database.eval(Code(group_function, {"ns": self.__collection_name,
-                                                          "keys": keys,
-                                                          "condition": condition,
-                                                          "initial": initial}))["result"]
+        return self.__database.eval(Code(group_function,
+                                         {"ns": self.__collection_name,
+                                          "keys": keys,
+                                          "condition": condition,
+                                          "initial": initial}))["result"]
 
     def __iter__(self):
         return self
@@ -509,9 +545,10 @@ class Collection(object):
         """This is only here so that some API misusages are easier to debug.
         """
         if "." not in self.__collection_name:
-            raise TypeError("'Collection' object is not callable. If you meant "
-                            "to call the '%s' method on a 'Database' object it "
-                            "is failing because no such method exists." %
+            raise TypeError("'Collection' object is not callable. If you "
+                            "meant to call the '%s' method on a 'Database' "
+                            "object it is failing because no such method "
+                            "exists." %
                             self.__collection_name)
         raise TypeError("'Collection' object is not callable. If you meant to "
                         "call the '%s' method on a 'Collection' object it is "
