@@ -177,8 +177,6 @@ class TestCollection(unittest.TestCase):
                           (u"ns", u"pymongo_test.test"),
                           (u"key", SON([(u"hello", 1)]))]) in list(db.system.indexes.find({"ns": u"pymongo_test.test"})))
 
-
-
     def test_index_info(self):
         db = self.db
         db.test.drop_indexes()
@@ -197,6 +195,37 @@ class TestCollection(unittest.TestCase):
         self.assert_(("world", ASCENDING) in db.test.index_information()["hello_-1_world_1"])
         self.assert_(len(db.test.index_information()["hello_-1_world_1"]) == 2)
 
+    def test_fields_list_to_dict(self):
+        f = self.db.test._fields_list_to_dict
+
+        self.assertEqual(f(["a", "b"]), {"a": 1, "b": 1})
+        self.assertEqual(f(["a.b.c", "d", "a.c"]), {"a": {"b": {"c": 1}, "c": 1}, "d": 1})
+        self.assertEqual(f(["a.c", "d", "a.b.c"]), {"a": {"b": {"c": 1}, "c": 1}, "d": 1})
+        self.assertEqual(f(["a.b.c", "d", "a.b.d"]), {"a": {"b": {"c": 1, "d": 1}}, "d": 1})
+
+    def test_field_selection(self):
+        db = self.db
+        db.drop_collection("test")
+
+        doc = {"a": 1, "b": 5, "c": {"d": 5, "e": 10}}
+        db.test.insert(doc)
+
+        self.assertEqual(db.test.find({}, ["_id"]).next().keys(), ["_id"])
+        self.assertEqual(set(db.test.find({}, ["a"]).next().keys()), set(["a", "_id"]))
+        self.assertEqual(set(db.test.find({}, ["b"]).next().keys()), set(["b", "_id"]))
+        self.assertEqual(set(db.test.find({}, ["c"]).next().keys()), set(["c", "_id"]))
+        self.assertEqual(db.test.find({}, ["a"]).next()["a"], 1)
+        self.assertEqual(db.test.find({}, ["b"]).next()["b"], 5)
+        self.assertEqual(db.test.find({}, ["c"]).next()["c"], {"d": 5, "e": 10})
+
+        # TODO some commented out tests due to what appears to be a bug in the server
+#         self.assertEqual(db.test.find({}, ["c.d"]).next()["c"], {"d": 5})
+#         self.assertEqual(db.test.find({}, ["c.e"]).next()["c"], {"e": 10})
+
+        self.assertEqual(set(db.test.find({}, ["b", "c.e"]).next().keys()), set(["b", "c", "_id"]))
+        self.assertEqual(db.test.find({}, ["b", "c.e"]).next()["b"], 5)
+#         self.assertEqual(db.test.find({}, ["b", "c.e"]).next()["c"], {"e": 10})
+
     def test_options(self):
         db = self.db
         db.drop_collection("test")
@@ -285,6 +314,28 @@ class TestCollection(unittest.TestCase):
 
         self.assertRaises(TypeError, iterate)
 
+    def test_invalid_key_names(self):
+        db = self.db
+        db.test.remove({})
+
+        db.test.insert({"hello": "world"})
+        db.test.insert({"hello": {"hello": "world"}})
+
+        self.assertRaises(InvalidName, db.test.insert, {"$hello": "world"})
+        self.assertRaises(InvalidName, db.test.insert, {"hello": {"$hello": "world"}})
+
+        db.test.insert({"he$llo": "world"})
+        db.test.insert({"hello": {"hello$": "world"}})
+
+        self.assertRaises(InvalidName, db.test.insert, {".hello": "world"})
+        self.assertRaises(InvalidName, db.test.insert, {"hello": {".hello": "world"}})
+        self.assertRaises(InvalidName, db.test.insert, {"hello.": "world"})
+        self.assertRaises(InvalidName, db.test.insert, {"hello": {"hello.": "world"}})
+        self.assertRaises(InvalidName, db.test.insert, {"hel.lo": "world"})
+        self.assertRaises(InvalidName, db.test.insert, {"hello": {"hel.lo": "world"}})
+
+        db.test.update({"hello": "world"}, {"$inc": "hello"})
+
     def test_insert_multiple(self):
         db = self.db
         db.drop_collection("test")
@@ -315,8 +366,23 @@ class TestCollection(unittest.TestCase):
         db.test.save({"hello": "world"})
         self.assert_(db.error())
 
+    def test_index_on_subfield(self):
+        db = self.db
         db.drop_collection("test")
 
+        db.test.insert({"hello": {"a": 4, "b": 5}})
+        db.test.insert({"hello": {"a": 7, "b": 2}})
+        db.test.insert({"hello": {"a": 4, "b": 10}})
+        self.failIf(db.error())
+
+        db.drop_collection("test")
+        db.test.create_index("hello.a", ASCENDING, unique=True)
+
+        db.test.insert({"hello": {"a": 4, "b": 5}})
+        db.test.insert({"hello": {"a": 7, "b": 2}})
+        db.test.insert({"hello": {"a": 4, "b": 10}})
+        self.assert_(db.error())
+
     def test_safe_insert(self):
         db = self.db
         db.drop_collection("test")
@@ -325,7 +391,7 @@ class TestCollection(unittest.TestCase):
         a = {"hello": "world"}
         db.test.insert(a)
         db.test.insert(a)
-        self.assertEqual(db.error()["err"], "E11000 duplicate key error")
+        self.assert_("E11000" in db.error()["err"])
 
         self.assertRaises(OperationFailure, db.test.insert, a, safe=True)
 
